/* filename:    sunmh.g
** description: The program generates draws from the posterior
**              distribution via the random walk Metropolis Hastings Algorithm
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g
#include c:\projects\active\sunspots\gauss1\sunspec.g


/******************************************************** 
**  Configure the Metropolis Algorithm
*/
mhrun   = "1";
cc0     = 1;
cc      = 0.15;
nblocks = 500;
nsim    = 1000;
opath = parapath $+ "\\mhrun" $+ mhrun;

_bounds =
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 100     )|
    (1E-5 ~ 100     )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (-0.99999 ~ 0.99999)|
    (-100 ~ 100)|
    (-100 ~ 100)|
    (-100 ~ 100)|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     );

/******************************************************** 
** Import data on output, inflation, nominal interest rate
** series (nobs,3)
*/
#include c:\projects\active\sunspots\gauss1\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0    = 1;

/******************************************************** 
** Import the information for the prior density
*/
priorfile = "m" $+ lprior $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,subT];

/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;
load path = ^priopath para_names;

"Model            " $lmodel;
"Sample           " $dataselstr;
"Prior            " $lprior;
"Fix Para         " $subTstr;
"MHrun Index      " $mhrun;
"Scaling of Hess  " cc;
"";
"===================================";
"";
"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);
{lnpost0, lnpy0} = fcn(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "mu";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,npara );
closeall fhmult;

lhess = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "hs";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

SIGPROPdim = sumc(para_maskinv[.,subT]);

{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/
ostat = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
opara = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
statname = { POST LIKE PROP PROPPOST REJECT };

create fhstat=^ostat with ^statname, 5, 8;
create fhpara=^opara with ^para_names, npara, 8;

/******************************************************** 
**              Metropolis Hastings Algorithm
*/

/* Initialize PARANEW
*/
valid = 0;
do until valid == 1;
   paranew   = para + cc0*(sigscale*rndn(npara,1));
   paranew   = paranew.*para_maskinv[.,subT] + para_fix[.,subT].*para_mask[.,subT];

   {postnew, likenew} = fcn(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;

endo;   


iblock = 1;
do until iblock > nblocks;

   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1;
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
   else;
      j=1;
   endif;      

   do until j > nsim;

     paranew          = paraold + cc*(sigscale*rndn(npara,1));
     paranew   = paranew.*para_maskinv[.,subT] + para_fix[.,subT].*para_mask[.,subT];

     {postnew, likenew} = fcn(paranew);
     propdens         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                        -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
     propsim[j,1]     = propdens;
     proppostsim[j,1] = postnew;

     r = minc(1 | exp( postnew - postold));
     if rndu(1,1) < r;
       /* Accept proposed jump
       */
       postsim[j,1] = postnew;
       likesim[j,1] = likenew;
       parasim[j,.] = paranew';
       paraold = paranew;
       postold = postnew;
       likeold = likenew;
     else;
       /* Reject proposed jump
       */
       likesim[j,1] = likeold;
       postsim[j,1] = postold;
       parasim[j,.] = paraold';
       rej[j] = 1;
     endif;

     /*
     locate 1,1;
     "Block" iblock "of" nblocks;
     "Simulation step" j "of" nsim;
     "Proposal Dens  " propsim[j,1];
     "Parameters     "; parasim[j,.]; 
     */

     j = j+1;
    endo;

   locate 1,1;
   "Block" iblock "of" nblocks;
   "Rejection: r / rej / perct " r rej[nsim] sumc(rej)/nsim;
   "Likelihood     " likesim[nsim,1];
   "Posterior      " postsim[nsim,1];
   "Parameters     "; parasim[nsim,.]; 

    wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
    wr = writer(fhpara,parasim);

    iblock = iblock+1;
endo;  

closeall fhstat, fhpara;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, para,
      parabd_ind1, parabd_ind2;

    parabd_ind1 = para .> _bounds[.,1];
    parabd_ind2 = para .< _bounds[.,2];
    parabd_ind1 = delif(parabd_ind1,para_mask[.,subT]);
    parabd_ind2 = delif(parabd_ind2,para_mask[.,subT]);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);

       {lnpY,retcode,obsmean,obsvar} = evalcgg(para,mspec,YY); 
       lnpy    = real(lnpy);
       lnprio  = priocgg( para, pmean, pstdd, pshape);
       lnpost  = lnpy + real(lnprio);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
        
retp(lnpost,lnpy);  
endp;

#include c:\projects\active\sunspots\gauss1\sunmod.src;
#include c:\projects\active\sunspots\gauss1\sunlh.src;
