#include <oxstd.h>
#include <oxdraw.h>
#include <oxprob.h>
#include <oxfloat.h>
#import <maximize>


decl n;

const decl p_halpha=0.4, p_nudot=3, p_r=3, m=3;
const decl p_s2=3, p_nu=3,p_Hbeta=.01,p_Halpha=.01;

const decl loops=10000;

const decl Gibbsburnin=2000;
const decl Gibbsloops=5000;

decl alpha,beta,h,mpis,malphas,mhs,mstates;
decl x,y,x_y;

decl NLRgamma,NLRh;
decl XpX,Xpy,X;

getGibbs()
{
	decl xys=zeros(m,5);
	decl del=(-beta)|1;
	decl e2s=zeros(m,1);
	for(decl mx=0;mx<m;mx++){
		decl sxy=selectifr(x_y,mstates.==mx);
		if(rows(sxy)>0){
			xys[mx][0]=rows(sxy);
			xys[mx][1]=sumc(sxy[][0]);
			xys[mx][2:3]=sxy[][0]'sxy[][0:1];
			xys[mx][4]=sumc(sxy[][1]);		
			e2s[mx]=sumsqrc(sxy*del-(alpha+malphas[mx]));
		}
	}
	mhs=ranchi(m,1,p_nudot+xys[][0])./(p_nudot+h*e2s);
	h=ranchi(1,1,p_nu+m+n)/(p_s2+p_halpha*sumsqrc(malphas)+mhs'e2s);

	mpis=randirichlet(1,p_r+xys[][0])';
	mpis|=(1-sumc(mpis));
	
	decl P_Hgamma=zeros(m+2,m+2);
	xys.*=(h*mhs);
	P_Hgamma[0:1][0:1]=unvech(sumc(xys[][0:2]))+p_Hbeta*unit(2);
	P_Hgamma[2:][0:1]=xys[][0:1];
	P_Hgamma[0:1][2:]=xys[][0:1]';
	P_Hgamma[2:][2:]=diag(xys[][0])+p_halpha*h*unit(m);

	decl WQ=sumc(xys[][4])|sumc(xys[][3])|xys[][4];

	decl P_Hgammai=invertsym(P_Hgamma);
	decl gamma=choleski(P_Hgammai)*rann(m+2,1)+P_Hgammai*WQ;
	beta=gamma[1]; alpha=gamma[0]; malphas=gamma[2:];

	decl w=cumulate((mpis.*sqrt(mhs)).*exp((-.5*h*mhs).*sqr(y'-alpha-beta*x'-malphas)));
	mstates=sumc(w.<(ranu(1,n).*w[m-1][]))';

// Now Normal Linear Regression Model draw

	decl Hi=invertsym(p_Hbeta*unit(2)+NLRh*XpX);
	NLRgamma=Hi*(NLRh*Xpy)+choleski(Hi)*rann(2,1);
	NLRh=ranchi(1,1,p_nu+n)/(p_s2+sumsqrc(y-X*NLRgamma));
}

decl mindata;

decl linexb=2;

linex(const x)
{
	return min(exp(linexb*sqrt(n)*x)-linexb*sqrt(n)*x-1,30);
}

flinex(const a, const adLoss, const dum1, const dum2)
{
	decl d=(sqrt(n)*linexb)*(a-mindata);
	adLoss[0]=-meanc(setbounds(exp(d)-d-1,0,30));	
	return 1;
}

minlinex()
{
	decl loss,a=0;
	MaxBFGS(flinex,&a,&loss,0,1);
	return a;		
}

decl HPDc=40;
fHPD(const a, const adLoss, const dum1, const dum2)
{
	decl al=a[0];
	decl au=al+exp(a[1]);
	adLoss[0]=-meanc(setbounds(
			sqrt(n)*(au-al+HPDc*(mindata.<al).*(al-mindata)+HPDc*(mindata.>au).*(mindata-au))
			,0,80));	
	return 1;
}


minHPD()
{
	decl loss,a=quantilec(mindata,(1/HPDc)~(1-1/HPDc));
	MaxBFGS(fHPD,&a,&loss,0,1);
	return a[0]|(a[0]+exp(a[1]));		
}

HPD(const a)
{
	return min(sqrt(n)*(a[1]-a[0]+HPDc*(a[0]>0)*a[0]+HPDc*(a[1]<0)*(-a[1])),80);
}

decl fp1, fp2,fmu1,fmu2,fsig1,fsig2;

DGPnorm()
{
	y=rann(n,1);
}
DGPmix()
{
	decl eps=rann(n,1),u=ranu(n,1);
	y=(u.<fp1).? (fmu1+fsig1*eps) .: (fmu2+fsig2*eps);
}
DGPasy()
{
	decl eps=rann(n,1),u=ranu(n,1);
	y=(u.<fp1).? (fmu1+fsig1*eps) .: (fmu2+fsig2*eps);
	y=(1-2*(x.>0)).*((u.<fp1).? (fmu1+fsig1*eps) .: (fmu2+fsig2*eps)); 
}

const decl nloss=2;

main()
{
	format(5000);
	ranseed(230);

	decl args=arglist(),rs;
	if (rows(args)>1) sscan(args[1],"%d",&rs); else rs=1;

	decl DGPlc=idiv(rs-1,3);
	decl nlc=imod(rs-1,3);
	
	decl swrann=rann(Gibbsloops,1);
	
	fp1=.8, fp2=1-fp1;
	fmu1=-.25, fmu2= -fmu1*fp1/fp2;
	fsig1= .75, fsig2=sqrt( (1-fp1*(sqr(fsig1)+sqr(fmu1)))/fp2-sqr(fmu2) );

	decl DGPlist={DGPnorm,DGPmix,DGPasy};
	decl nlist={50,200,800};
	decl allout=new array[nloss];
	decl alloutraw=allout;
	decl alloutstd;
	allout[]=zeros(6,6);
	alloutstd=allout;
	alloutraw[]=zeros(9,6);
	
	decl time=timer();
	
//	for(decl nlc=0;nlc<rows(nlist);nlc++){
	n=nlist[nlc];

//	for(decl DGPlc=0;DGPlc<rows(DGPlist);DGPlc++){

	for(decl hetlc=0;hetlc<2;hetlc++){
	
	decl risk,cumrisk,stdrisk;
	risk=cumrisk=zeros(nloss,3);
	decl risklist=zeros(loops,nloss*3);
	
	for(decl mlc=0;mlc<loops;mlc++){

	ranseed(mlc);
	x=rann(n,1);

	DGPlist[DGPlc]();
	if(hetlc) y.*=(.5+fabs(x))/sqrt(3.25+2*sqrt(2/M_PI));

	beta=alpha=0;
	mpis=ones(m,1)/m;
	mstates=floor(m*ranu(n,1));
	h=1;
	mhs=ones(m,1);
	malphas=zeros(m,1);
	x_y=x~y;

	NLRgamma=zeros(2,1);
	NLRh=1;
	X=ones(n,1)~x;
	XpX=X'X;
	Xpy=X'y;
		
	for(decl lc=0;lc<Gibbsburnin;lc++)
		getGibbs();
		
	decl betas=zeros(Gibbsloops,2);
	for(decl lc=0;lc<Gibbsloops;lc++){
		getGibbs();
		betas[lc][]=beta~NLRgamma[1];
		}

	mindata=betas[][1];
	risk[0][0]=linex(minlinex());
	risk[1][0]=HPD(minHPD());

	mindata=betas[][0];
	risk[0][2]=linex(minlinex());
	risk[1][2]=HPD(minHPD());

	decl bdum,xpxi,X=x~ones(n,1);
	olsc(y,X,&bdum,&xpxi);
	decl sw=sumsqrc(x.*(y-X*bdum))*sqr(xpxi[0][0]);

	mindata=bdum[0]+sqrt(sw)*swrann;
	risk[0][1]=linex(minlinex());
	risk[1][1]=HPD(minHPD());

	cumrisk+=risk;
	risklist[mlc][]=vecr(risk)';

	}
	print(.01*(timer()-time));
	print(quantilec(risklist)|meanc(risklist)|sqrt(varc(risklist)/loops));
	cumrisk/=loops;

	for(decl j=0;j<nloss;j++){
		allout[j][nlc*2:nlc*2+1][hetlc*3+DGPlc]=cumrisk[j][<1,2>]'/cumrisk[j][0];
		alloutraw[j][nlc*3:nlc*3+2][hetlc*3+DGPlc]=cumrisk[j][0:2]';
		alloutstd[j][nlc*2:nlc*2+1][hetlc*3+DGPlc]=sqrt(varc(risklist[][3*j+<1,2>]/cumrisk[j][0]-cumrisk[j][<1,2>].*risklist[][3*j]/sqr(cumrisk[j][0]))'/loops);
	}
	print(allout,alloutraw,alloutstd);
	}
	savemat(sprint("LRMC",rs,".mat"),allout[0]|allout[1]|alloutraw[0]|alloutraw[1]|alloutstd[0]|alloutstd[1]);
}