#include <oxstd.h>
#include <arma.h>
#include <oxfloat.h>
#include <oxprob.h>
#include <oxdraw.h>

decl n,k;

decl p_halpha=0.4, p_nudot=3, p_r=3, m=3;
decl p_s2=.01, p_nu=3,p_Hbeta,p_gamma;

const decl Gibbsburnin=10000;
const decl Gibbsloops=100000;

decl gamma,h,mpis,mhs,mstates;

decl X,y;

getGibbs()
{
	decl Zt=(mstates.==range(0,m-1));
	decl Wt=Zt~X;
	decl p_Hgamma=diagcat(p_halpha*h*unit(m),p_Hbeta);
	decl Qt=mhs[mstates];

	decl H_gammai=invertsym(p_Hgamma+h*Wt'(Qt.*Wt));
	gamma=choleski(H_gammai)*rann(m+k,1)+H_gammai*(p_Hgamma*p_gamma+h*Wt'(Qt.*y));
	decl eps=y-gamma[mstates]-X*gamma[m:];
	h=ranchi(1,1,p_nu+m+n)/(p_s2+p_halpha*sumsqrc(gamma[0:m-1])+sumc(Qt.*sqr(eps)));
	mpis=randirichlet(1,p_r+sumc(Zt))';
	mpis|=(1-sumc(mpis));
	mhs=(ranchi(1,m,p_nudot+sumc(Zt))./(p_nudot+h*sumc(Zt.*sqr(eps))))';
	decl w=cumulate((mpis.*sqrt(mhs)).*exp((-.5*h*mhs).*sqr(y'-gamma[0:m-1]-gamma[m:]'X')));
	mstates=sumc(w.<(ranu(1,n).*w[m-1][]))';


}

getGibbsNLR()
{
	decl Hi=invertsym(p_Hbeta+h*X'X);
	gamma=Hi*(p_Hbeta*p_gamma+h*X'y)+choleski(Hi)*rann(k,1);
	h=ranchi(1,1,p_nu+n)/(p_s2+sumsqrc(y-X*gamma));
}

kernel(decl eps)
{
	decl r=range(-.3,.3,.01);
	decl n=rows(eps);
	decl h=1.06*min(sqrt(varc(eps)),(<1,-1>*quantilec(eps,<.75,.25>))/(1.34))*n^(-.2);
	return r|sumc(densn((eps-r)/h))/(n*h);

}

main()
{
	format(5000);
	decl data=loadmat("elec198886.dat");
	data=data[][<7,8,9,2,3,4>];
	data=deleteifr(data,data[][<1,2,4,5>].<0);
	data[][1:2]./=sumr(data[][1:2]);		
	data[][4:5]./=sumr(data[][4:5]);		

	data=deleteifr(data,data[][<1,2,4,5>].==0);
	data=deleteifr(data,data[][<1,2,4,5>].>.95);
	println(rows(data));
	decl win=fabs(data[][3]).>0 .? data[][3].>0 .: (data[][1].>data[][2]);
	y=win .? data[][4] .: data[][5];
	X=fabs(data[][3])~(win.? data[][1] .: data[][2])~(2*win-1)~1;

	n=rows(X);
	k=columns(X);

	decl outtab=zeros(2*k,3);

	decl beta,xpxi;
	olsc(y,X,&beta,&xpxi);

	savemat("eps0.mat",kernel(selectifr(y-X*beta,X[][0])),1);
	savemat("eps1.mat",kernel(selectifr(y-X*beta,!X[][0])),1);
	print(kernel(selectifr(y-X*beta,X[][0])));
	print(kernel(selectifr(y-X*beta,!X[][0])));

//	exit(0);
	

	DrawDensity(0,selectifr(y-X*beta,X[][0])',"1",1);
	DrawDensity(1,selectifr(y-X*beta,!X[][0])',"0",1);
	ShowDrawWindow();
	
	decl Om=xpxi*outer(X.*(y-X*beta),<>,'o')*xpxi;
	outtab[][1]=vecr(beta~(sqrt(diagonal(Om))'));
	

	decl abeta,abeta2,astd;
	p_Hbeta=.01*unit(k);
	p_gamma=zeros(k+m,1);
	
	mpis=ones(m,1)/m;
	mstates=floor(m*ranu(n,1));
	h=p_nu/p_s2;
	mhs=ones(m,1);
	gamma=p_gamma;

	for(decl lc=0;lc<Gibbsburnin;lc++)
		getGibbs();
	abeta=abeta2=zeros(k,1);
	for(decl lc=0;lc<Gibbsloops;lc++){
		abeta+=gamma[m:];				
		abeta2+=sqr(gamma[m:]);
		getGibbs();
	}
	abeta/=Gibbsloops;
	abeta2/=Gibbsloops;
	astd=sqrt(abeta2-sqr(abeta));
	outtab[][2]=vecr(abeta~astd);

	p_gamma=zeros(k,1);
	gamma=p_gamma;
	h=p_nu/p_s2;

	for(decl lc=0;lc<Gibbsburnin;lc++)
		getGibbsNLR();
	abeta=abeta2=zeros(k,1);
	for(decl lc=0;lc<Gibbsloops;lc++){
		abeta+=gamma;				
		abeta2+=sqr(gamma);
		getGibbsNLR();
	}
	abeta/=Gibbsloops;
	abeta2/=Gibbsloops;
	astd=sqrt(abeta2-sqr(abeta));
	outtab[][0]=vecr(abeta~astd);

	print(outtab);

	
}
